@@ -13,7 +13,8 @@
 
 #include "getbits.h"
 
-#if (defined (WIN32) && !defined (__CYGWIN__))
+#if defined (WANT_WIN32_SOCKETS)
+#include <winsock2.h>
 #include <ws2tcpip.h>
 #endif
 
@@ -38,7 +39,7 @@
 	C: layer
 	D: CRC
 	E: bitrate
-	F:sampling rate
+	F: sampling rate
 	G: padding
 	H: private
 	I: channel mode
@@ -60,9 +61,11 @@
 	Even more, I'll allow varying crc bit.
 	11111111 11111110 00001101 00000000
 
-	(still unsure about this private bit)
+	Still unsure about this private bit... well, as Marcel pointed out:
+	The decoder should not care about this.
+	11111111 11111110 00001100 00000000
 */
-#define HDRCMPMASK 0xfffe0d00
+#define HDRCMPMASK 0xfffe0c00
 #define HDRSAMPMASK 0xc00 /* 1100 00000000, FF bits (sample rate) */
 
 /* bitrates for [mpeg1/2][layer] */
@@ -80,16 +83,10 @@ static const int tabsel_123[2][3][16] =
 	}
 };
 
-const long freqs[9] = { 44100, 48000, 32000, 22050, 24000, 16000 , 11025 , 12000 , 8000 };
+static const long freqs[9] = { 44100, 48000, 32000, 22050, 24000, 16000 , 11025 , 12000 , 8000 };
 
 static int decode_header(mpg123_handle *fr,unsigned long newhead);
 
-int read_frame_init(mpg123_handle* fr)
-{
-	if(frame_reset(fr) != 0) return -1;
-	return 0;
-}
-
 /* These two are to be replaced by one function that gives all the frame parameters (for outsiders).*/
 
 int frame_bitrate(mpg123_handle *fr)
@@ -105,7 +102,7 @@ long frame_freq(mpg123_handle *fr)
 #define free_format_header(head) ( ((head & 0xffe00000) == 0xffe00000) && ((head>>17)&3) && (((head>>12)&0xf) == 0x0) && (((head>>10)&0x3) != 0x3 ))
 
 /* compiler is smart enought to inline this one or should I really do it as macro...? */
-int head_check(unsigned long head)
+static int head_check(unsigned long head)
 {
 	if
 	(
@@ -146,7 +143,10 @@ static int check_lame_tag(mpg123_handle *fr)
 	int lame_offset = (fr->stereo == 2) ? (fr->lsf ? 17 : 32 ) : (fr->lsf ? 9 : 17);
 	/* At least skip the decoder delay. */
 #ifdef GAPLESS
-	if(fr->begin_s == 0) frame_gapless_init(fr, GAPLESS_DELAY, 0);
+	if(fr->p.flags & MPG123_GAPLESS)
+	{
+		if(fr->begin_s == 0) frame_gapless_init(fr, GAPLESS_DELAY, 0);
+	}
 #endif
 
 	if(fr->framesize >= 120+lame_offset) /* traditional Xing header is 120 bytes */
@@ -198,49 +198,66 @@ static int check_lame_tag(mpg123_handle *fr)
 				debug1("Xing: flags 0x%08lx", xing_flags);
 				if(xing_flags & 1) /* frames */
 				{
-					/*
-						In theory, one should use that value for skipping...
-						When I know the exact number of samples I could simply count in flush_output,
-						but that's problematic with seeking and such.
-						I still miss the real solution for detecting the end.
-					*/
-					fr->track_frames = (off_t) make_long(fr->bsbuf, lame_offset);
-					if(fr->track_frames > TRACK_MAX_FRAMES) fr->track_frames = 0; /* endless stream? */
-					#ifdef GAPLESS
-					/* if no further info there, remove/add at least the decoder delay */
-					if(fr->p.flags & MPG123_GAPLESS)
+					if(fr->p.flags & MPG123_IGNORE_STREAMLENGTH)
 					{
-						off_t length = fr->track_frames * spf(fr);
-						if(length > 1)
-						frame_gapless_init(fr, GAPLESS_DELAY, length+GAPLESS_DELAY);
+						if(VERBOSE3)
+						fprintf(stderr, "Note: Ignoring Xing frames because of MPG123_IGNORE_STREAMLENGTH\n");
 					}
-					#endif
-					if(VERBOSE3) fprintf(stderr, "Note: Xing: %lu frames\n", (long unsigned)fr->track_frames);
+					else
+					{
+						/*
+							In theory, one should use that value for skipping...
+							When I know the exact number of samples I could simply count in flush_output,
+							but that's problematic with seeking and such.
+							I still miss the real solution for detecting the end.
+						*/
+						fr->track_frames = (off_t) make_long(fr->bsbuf, lame_offset);
+						if(fr->track_frames > TRACK_MAX_FRAMES) fr->track_frames = 0; /* endless stream? */
+						#ifdef GAPLESS
+						/* if no further info there, remove/add at least the decoder delay */
+						if(fr->p.flags & MPG123_GAPLESS)
+						{
+							off_t length = fr->track_frames * spf(fr);
+							if(length > 1)
+							frame_gapless_init(fr, GAPLESS_DELAY, length+GAPLESS_DELAY);
+						}
+						#endif
+						if(VERBOSE3) fprintf(stderr, "Note: Xing: %lu frames\n", (long unsigned)fr->track_frames);
+					}
+
 					lame_offset += 4;
 				}
 				if(xing_flags & 0x2) /* bytes */
 				{
-					unsigned long xing_bytes = make_long(fr->bsbuf, lame_offset);					/* We assume that this is the _total_ size of the file, including Xing frame ... and ID3 frames...
-					   It's not that clearly documented... */
-					if(fr->rdat.filelen < 1)
-					fr->rdat.filelen = (off_t) xing_bytes; /* One could start caring for overflow here. */
+					if(fr->p.flags & MPG123_IGNORE_STREAMLENGTH)
+					{
+						if(VERBOSE3)
+						fprintf(stderr, "Note: Ignoring Xing bytes because of MPG123_IGNORE_STREAMLENGTH\n");
+					}
 					else
 					{
-						if((off_t) xing_bytes != fr->rdat.filelen && NOQUIET)
+						unsigned long xing_bytes = make_long(fr->bsbuf, lame_offset);					/* We assume that this is the _total_ size of the file, including Xing frame ... and ID3 frames...
+						   It's not that clearly documented... */
+						if(fr->rdat.filelen < 1)
+						fr->rdat.filelen = (off_t) xing_bytes; /* One could start caring for overflow here. */
+						else
 						{
-							double diff = 1.0/fr->rdat.filelen * (fr->rdat.filelen - (off_t)xing_bytes);
-							if(diff < 0.) diff = -diff;
+							if((off_t) xing_bytes != fr->rdat.filelen && NOQUIET)
+							{
+								double diff = 1.0/fr->rdat.filelen * (fr->rdat.filelen - (off_t)xing_bytes);
+								if(diff < 0.) diff = -diff;
 
-							if(VERBOSE3)
-							fprintf(stderr, "Note: Xing stream size %lu differs by %f%% from determined/given file size!\n", xing_bytes, diff);
+								if(VERBOSE3)
+								fprintf(stderr, "Note: Xing stream size %lu differs by %f%% from determined/given file size!\n", xing_bytes, diff);
 
-							if(diff > 1.)
-							fprintf(stderr, "Warning: Xing stream size off by more than 1%%, fuzzy seeking may be even more fuzzy than by design!\n");
+								if(diff > 1.)
+								fprintf(stderr, "Warning: Xing stream size off by more than 1%%, fuzzy seeking may be even more fuzzy than by design!\n");
+							}
 						}
-					}
 
-					if(VERBOSE3)
-					fprintf(stderr, "Note: Xing: %lu bytes\n", (long unsigned)xing_bytes);
+						if(VERBOSE3)
+						fprintf(stderr, "Note: Xing: %lu bytes\n", (long unsigned)xing_bytes);
+					}
 
 					lame_offset += 4;
 				}
@@ -531,7 +548,7 @@ init_resync:
 		off_t start = fr->rd->tell(fr);
 		if(ret<0){ debug("need more?"); goto read_frame_bad; }
 
-		debug2("doing ahead check with BPF %d at %li", fr->framesize+4, (long)start);
+		debug2("doing ahead check with BPF %d at %"OFF_P, fr->framesize+4, (off_p)start);
 		/* step framesize bytes forward and read next possible header*/
 		if((ret=fr->rd->skip_bytes(fr, fr->framesize))<0)
 		{
@@ -634,7 +651,7 @@ init_resync:
 
 					goto read_frame_bad;
 				}
-				if(VERBOSE3) debug3("resync try %li at 0x%"OFF_P", got newhead 0x%08lx", try, (off_p)fr->rd->tell(fr),  newhead);
+				if(VERBOSE3) debug3("resync try %li at %"OFF_P", got newhead 0x%08lx", try, (off_p)fr->rd->tell(fr),  newhead);
 
 				if(!fr->oldhead)
 				{
@@ -753,8 +770,9 @@ init_resync:
 		fr->mean_framesize = ((fr->mean_frames-1)*fr->mean_framesize+compute_bpf(fr)) / fr->mean_frames ;
 	}
 	++fr->num; /* 0 for first frame! */
-	debug4("Frame %li %08lx %i, next filepos=%"OFF_P, 
-	(long)fr->num, newhead, fr->framesize, (off_p)fr->rd->tell(fr));
+	debug4("Frame %"OFF_P" %08lx %i, next filepos=%"OFF_P, 
+	(off_p)fr->num, newhead, fr->framesize, (off_p)fr->rd->tell(fr));
+
 	/* save for repetition */
 	if(fr->p.halfspeed && fr->lay == 3)
 	{
